{ Poly.pas     v3.0 Copyright 5/1994  Adolfo Di Mare }

{$I ADT-ADH.inc}

UNIT Poly;

{ RESULTADO
  Implementacin el tipo abstracto de datos TPoly, que permite
  realizar operaciones algebraicas sobre polinomios de
  coeficientes reales y de grado maximo Poly.Max_Grado.

  Las operaciones implementadas son las siguientes :
  - SetTerm      - Degree    - Equal    -  Coef
  - Add          - Sub       - Multiply -  Minus
  - Zero         - To_STRING


  - Init    - Copy   - Load  - OK
  - Clear   - Move   - Store - Fix
  - Done    - Equal                 }

{ REQUIERE
  Que el grado de los polinomios no sea mayor que la constante
  Poly.Max_Grado }

{ PROGRAMADOR          ADH ==> Adolfo Di Mare        }

{ COLABORADORES        Varios estudiantes del curso
                       CI-1201 Programacin II

                       RSH ==> Roberto Sala Hidalgo  }

{[])=============([]}
{[]}  INTERFACE  {[]}
{[])=============([]}

CONST
  Max_Grado = 25; { mximo grado de un polinomio }

TYPE
  TPoly = OBJECT
                                  {$IFDEF Use_PRIVATE}
  PRIVATE                         {$ENDIF Use_PRIVATE}
    _grad : WORD; { numerador   }
    _coef : ARRAY[0..Max_Grado] OF REAL;

     { truco especial de Adolfo para Programacin I }
    _mrk : INTEGER; { marca de inicializacin }

    PROCEDURE Adjust_Degree;
                                  {$IFDEF Use_PUBLIC}
  PUBLIC                          {$ENDIF Use_PUBLIC}
    PROCEDURE Init; { Constructor }
    PROCEDURE Clear;
    PROCEDURE Done; { Destructor  }

    PROCEDURE Copy( VAR o: TPoly);
    PROCEDURE Move( VAR o: TPoly);

    FUNCTION  Equal (VAR o: TPoly): BOOLEAN;
    FUNCTION  Less  (VAR o: TPoly): BOOLEAN;

    FUNCTION  Ok: BOOLEAN;
    PROCEDURE Fix;

    PROCEDURE Load  (VAR F: FILE);
    PROCEDURE Store (VAR F: FILE);

    PROCEDURE Read  (VAR F: TEXT);
    PROCEDURE Print (VAR F: TEXT);


    PROCEDURE Zero;
    PROCEDURE SetTerm   (c : REAL; d : WORD);
    FUNCTION  Degree    : WORD;
    FUNCTION  Coef      (exp : WORD) : REAL;

    PROCEDURE Add       (VAR o: TPoly);
    PROCEDURE Substract (VAR o: TPoly);

    PROCEDURE Multiply  (VAR o: TPoly);
    PROCEDURE Divide    (VAR o: TPoly);
    PROCEDURE Modulo    (VAR o: TPoly);
    PROCEDURE Minus;

    FUNCTION  Eval      (x : REAL): REAL;

    FUNCTION  Is_ZERO       : BOOLEAN;
    FUNCTION  To_STRING     : STRING;
    PROCEDURE From_STRING (s: STRING);

  END;  { TPoly }

  FUNCTION CheckSyntax(VAR S: STRING) : WORD;

{[])==================([]}
{[]}  IMPLEMENTATION  {[]}
{[])==================([]}

{ Un polinomio es una arreglo de Max_Grado nmeros reales que
  contiene, en su entrada "i", el coeficiente del i-simo
  trmino del polinomio.
  - El valor del campo SELF._grad indica el grado del
    polinomio.
  - Los trminos superiores a ._grad no estn inicializados
    en cero.
  - El polinomio cero se representa con
    ._coef[0] = 0 y ._grad = 0.

  Por ejemplo, el polinomio x^3 + 2x^2 se represenata as:

  polinomio                grado
  x^3 + 2x^2                 3

    0   1   2   3   4   5           Max_Grado
  +---+---+---+---+---+---+ ... +---+---+---+
  | 0 | 0 | 2 | 1 | $ | ! | ... | @ | * | ? |
  +---+---+---+---+---+---+ ... +---+---+---+
                ^
                |
                +----- polinomio._grad = 3  }

VAR
  Initialized_TPoly : INTEGER;
  Destroyed_TPoly   : INTEGER;

TYPE
  String13 = STRING[13];

PROCEDURE Check_Ok(               { LOCAL-ADH }
  {+} VAR r:  TPoly;     { SELF }
  {+}     op: String13   { operacin invocadora }
);
{ RESULTADO
  Verifica, por medio de la operacin Ok(), que la
  invariante del Tipo Abstracto de Datos se cumple.
  - Si este no es el caso, despliega un pequeo
    mensaje de error que incluye a "op" que es el
    nombre de la operacin que invoc a este
    procedimiento.
  - No es usual que los ADTs incluyan este tipo de
    operacin; se incluye aqu por un fin meramente
    didctico. El verificar la invariante del ADT
    constantemente en tiempo de ejecucin es, en
    general, muy costoso, y a veces es imposible.  }
BEGIN { Check_Ok }
  { este cdigo ayuda a que el programador bisoo }
  { encuentre errores en sus primeros programas   }
  IF NOT r.Ok THEN BEGIN
    r.Print(OUTPUT);
    Write(OUTPUT, '.', op:1, ': ');
    IF r._mrk <> Initialized_TPoly  THEN BEGIN
      WriteLn(OUTPUT, 'Objeto no inicializado');
    END
    ELSE IF r._mrk = Destroyed_TPoly THEN BEGIN
      WriteLn(OUTPUT, 'Objeto ya destruido');
    END
    ELSE BEGIN
      WriteLn(OUTPUT, 'Denominador invlido');
    END;
  END;
END;  { Check_Ok }

PROCEDURE TPoly.Adjust_Degree;            { LOCAL-ADH }
{ RESULTADO
  Reduce el grado de SELF hasta que encuentra un
  trmino cuyo coeficiente sea diferente de cero. }
BEGIN { TPoly.Adjust_Degree }
  WHILE
    (_grad>0)
     AND
    (_coef[_grad] = 0.0)
  DO BEGIN
    DEC(_grad);
  END;
END;  { TPoly.Adjust_Degree }


PROCEDURE TPoly.Init;        { EXPORT }         { ADH }
{ RESULTADO
  Constructor del tipo de datos, que sirve para
  inicializar a la variable "SELF".
  - El constructor es la nica operacin del Tipo
    Abstracto de Datos que opera sobre una variable
    que no ha sido inicializada; todas las otras
    operaciones requieren que la variable est
    adecuadamente inicializada.
  - Si se reinicializa a una variable que oa fue
    inicializada, lo que puede ocurrir es que quede un
    hueco en la memoria dinmica; por  supuesto, esto
    ocurre slo para aquellos Tipos Abstractos de
    Datos que utilizan memoria dinmica. Este error
    disminuoe la cantidad de memoria dinmica
    disponible para el programa, pero en general no
    tiene otras consecuencias negativas.
  - En buena prctica, a cada invocacin de Init()
    debe corresponder exactamente una del destructor
    Done(). }
{ RESULTADO
  Inicializa al polinomio SELF.
  - Le asigna a SELF el valor del polinomio cero. }
BEGIN { TPoly.Init }
  SELF._coef[0] := 0.0;
  SELF._grad    :=   0;

  { este cdigo ayuda a que el programador bisoo }
  { encuentre errores en sus primeros programas   }
   _mrk := Initialized_TPoly;
END;  { TPoly.Init }

PROCEDURE TPoly.Clear;       { EXPORT }         { ADH }
{ RESULTADO
  Deja a "SELF" en el mismo estado que lo dej el
  constructor Init() al inicializarlo.
  - A diferencia de Init(), esta operacin requiere
    que "SELF" sea una variable adecuadamente
    inicializada. }
BEGIN { Clear }
  { cdigo de ayuda para el programador bisoo }
  Check_Ok(SELF, 'Clear'); { plqp }
    
  SELF._coef[0] := 0.0;
  SELF._grad    :=   0;
END;  { Clear }

PROCEDURE TPoly.Done;        { EXPORT }         { ADH }
{ RESULTADO
  Destruye totalmente a "SELF".
  - Devuelve al sistema de memoria dinmica toda la
    memoria asociada a "SELF".
  - En general, la destruccin de un objeto consiste
    en devolverle al manejador de memoria dinmica
    toda la memoria dinmica que usa, y cerrar sus
    archivos.
  - "SELF" queda inusable, hasta el punto de que la
    nica manera de volver a utilizarlo es
    reinicializndolo por medio de Init().
  - Si se usa el paquete de excepciones, al destruir
    el objeto tambin se le desliga de su contexto de
    excepcin, por lo que si se reinicializa de nuevo
    con Init() el resultado ser que quede ligado al
    contexto actual de excepcin, y no al que estaba
    anteriormente. }
BEGIN { TPoly.Done }
  Check_Ok(SELF, 'Done');

  _coef[0] := 0.0;
  _grad    := 0;

  _mrk := Destroyed_TPoly;

{ En realidad, como este ADT no usa memoria dinmica,
  el trabajo de la operacin Done() debera ser nulo.
  Sin embargo, lo que se hace es marcar el valor
  almacenado en el Rep de forma que si luego se trata
  de destruir o usar esta misma variable, entonces
  la rutina Check_Ok() pueda detectarlo.
  - Esto ayuda a los estudiantes a depurar sus
    programas, pues tienen la maa de destruir ms
    de una vez un objeto, o de usarlo despus de que
    ha diso destruido.
  - Por eso es que todas las operaciones de Elem.pas
    comienzan invocando a Check_Ok(), para detectar lo
    ms pronto posible que se estn usando instancias
    que han sido destruidas. }
END;  { TPoly.Done }


PROCEDURE TPoly.Copy(        { EXPORT }         { ADH }
  {+} VAR o: TPoly           { qu copiar }
);
{ RESULTADO
  Copia todo el contenido de "o" sobre "SELF".
  - El valor anterior de "SELF" se pierde.
  - Si "SELF" es "o" entonces su valor no cambia.
  - "o" mantiene su valor anterior. }
VAR
  i : WORD;
BEGIN { TPoly.Copy }
  Check_Ok(SELF, 'Copy');
  Check_Ok(o,    'Copy');

  IF o._grad < (Poly.Max_Grado DIV 2) THEN BEGIN
    FOR i := 0 TO o._grad DO BEGIN
      _coef[i] :=  o._coef[i];
    END;
    _grad := o. _grad;
  END
  ELSE BEGIN
    { hace copia bit por bit slo cuando el grado de }
    { "o" es muy grande, pues es en este caso copiar }
    { cada coeficiente por separado es ms lerdo.    }
    SELF := o;
  END;
END;  { TPoly.Copy }

PROCEDURE TPoly.Move(        { EXPORT }         { ADH }
  {?} VAR o: TPoly           { qu trasladar }
);
{ RESULTADO
  Traslada el valor de "o" a "SELF".
  - El valor anterior de "SELF" se pierde.
  - El nuevo valor de "SELF" es el que "o" tuvo.
  - "o" pasa a quedar en el estado en que la dej
    Init() cuando la inicializ.
  - Si "SELF" es "o" entonces su valor no cambia.

  - Esta operacin es similar a Copy(), pero difiere
    en que para algunos ADTs que usan memoria dinmica
    puede implementarse mucho ms eficientemente que
    Copy().

  - Copy() es ms rpido que Move(). }
BEGIN { TPoly.Move }
  Check_Ok(SELF, 'Move');
  Check_Ok(o,    'Move');

  Copy(o);
  o.Clear;
END;  { TPoly.Move }

FUNCTION  TPoly.Equal(       { EXPORT }         { ADH }
  {+} VAR o: TPoly           { qu comparar }
) {>>>>>>>}: BOOLEAN;        { Son o no iguales }
{ RESULTADO
  Devuelve TRUE si SELF = o y FALSE en otro caso.
  - Si tienen el mismo valor, y aunque no ocupen las
    mismas posiciones memoria, el resultado es TRUE. }
VAR
   i : WORD;
BEGIN { TPoly.Equal }
  Check_Ok(SELF, 'Equal');
  Check_Ok(o,    'Equal');

  IF _grad <> o._grad THEN BEGIN
    Equal := FALSE;
    EXIT;
  END;

  FOR i := 0 TO _grad DO BEGIN
    IF _coef[i] <> o._coef[i] THEN BEGIN
      Equal := FALSE;
      EXIT;
    END;
  END;

  Equal := TRUE;
END;  { TPoly.Equal }

FUNCTION  TPoly.Less(        { EXPORT }         { ADH }
  {+} VAR o: TPoly           { qu comparar }
) {>>>>>>>}: BOOLEAN;        { FALSE siempre }
{ RESULTADO
  Retorna FALSE siempre por que no es posible
  establecer un orden total entre polinomios. }
BEGIN { TPoly.Less }
  Check_Ok(SELF, 'Less');

  Less := FALSE;
END;  { TPoly.Less }

FUNCTION  TPoly.OK: BOOLEAN; { EXPORT }         { ADH }
{ RESULTADO
  Verifica que la invariante del ADT sea verdadera,
  o sea que "SELF" est construdo adecuadamente. }
{ REQUIERE
  - Si "SELF" est bien construido, entonces esta
    funcin retornar TRUE. Si no, retorna FALSE. }
BEGIN { TPoly.Ok }
  IF _grad > Max_Grado THEN BEGIN
    OK := FALSE;
    EXIT;
  END;

  OK :=
    (
      (_grad = 0) OR ( (_grad > 0) AND (_coef[_grad]<>0.0) )
    )
       AND
    (_mrk = Initialized_TPoly);

{ NOTA
  En general, la operacin Ok() no es tan simple de
  implementar, y muchas veces NO es posible
  implementarla adecuadamente, principalmente si el
  Rep del ADT usa memoria dinmica.
  - Por otro lado, Fix() puede implementarse
    simplemente invocando a Init(), pero puede que
    esto no sea lo ms eficiente.
  - No es usual que los ADTs incluyan campos como
    "_mrk". Como este ADT se usa para aprender
    programacin, entonces se le ha incluido este
    campo. }
END;  { TPoly.Ok }

PROCEDURE TPoly.Fix;         { EXPORT }         { ADH }
{ RESULTADO
  Si la invariante del ADT no se cumple, entonces lo
  arregla para que si se cumpla. En general, el
  resultado es dejarlo en el mismo estado que lo
  dejara Clear(). }
{ REQUIERE
  Para muchos ADTs que usan memoria dinmica es
  imposible implementar correctamente, de una manera
  segura, esta operacin, pues en general no es
  posible arreglar un ADT que usa memoria dinmica si
  hay punteros perdidos.
  - Esta operacin se incluye por completitud. }
BEGIN { TPoly.Fix }
  IF NOT Ok THEN BEGIN
    Clear;
  END;
END;  { TPoly.Fix }

PROCEDURE TPoly.Load(        { EXPORT }         { ADH }
  {?} VAR F: FILE            { de donde leer }
);
{ RESULTADO
  Lee del archivo "F" el contenido de "SELF".
  - El archivo queda posicionado exactamente en el
    BYTE siguiente a la ultima posicin grabada.
  - El contenido anterior de "SELF" se pierde. }
{ REQUIERE
  - F debe ser un archivo sin tipo.
  - F debe haber sido grabado por medio de Store().
  - F debe estar abierto.
  - F debe contener el valor "SELF" completo. }
{ EJEMPLO
  - F debe abrirse usando uno de los comandos:
    Assign(F,"nombre");    Assign(F,"nombre");
     ...                o   ...
    Reset(F,1);            ReWrite(F,1);
    pues de lo contrario Turbo Pascal usar un tamao
    incorrecto al leer o grabar. }
BEGIN { TPoly.Load }
  Check_Ok(SELF, 'Load');

  { Clear; }
  BlockRead(F, SELF, SizeOf(SELF));
END; { TPoly.Load }

PROCEDURE TPoly.Store(       { EXPORT }         { ADH }
  {?} VAR F: FILE            { adonde grabar }
);
{ RESULTADO
  Graba en el archivo F el contenido de SELF.
  - F debe ser un archivo sin tipo, y el grabado se
    hace secuencialmente, de forma que Load() pueda
    luego reconstruir SELF leyendo F.
  - El archivo queda posicionado exactamente en el
    BYTE siguiente a la ltima posicin grabada para
    SELF. }
{ REQUIERE
  - F debe ser un archivo sin tipo.
  - F debe estar debidamente abierto.
  - F debe tener espacio suficiente para grabar SELF
    completo. De otra manera, el programa se cancelar
    en tiempo de ejecucin.
  - F debe abrirse usando uno de los comandos:
    Assign(F,"nombre");    Assign(F,"nombre");
     ...                o   ...
    Reset(F,1);            ReWrite(F,1);
    pues de lo contrario Turbo Pascal usar un tamao
    incorrecto al leer o grabar. }
BEGIN { TPoly.Store }
  Check_Ok(SELF, 'Store');

  System.BlockWrite(F, SELF, SizeOf(SELF));
END; { TPoly.Store }

FUNCTION CheckSyntax(        {  EXPORT }        { RSH }
  {+} VAR S: STRING          { hilera a chequear }
) {>>>>>>>} : WORD;
{ RESULTADO
  Retorna el valor CERO (0) cuando la hilera puede ser
  exitosamente leida con TPoly.Read().
  - Si hay algn error, entonces el valor retornado
    es la posicin dentro de la hilera "S" en donde
    est localizado el problema.
  - Un ejemplo de sintaxis correcta es:
    ( + 3.5 x^ 2 - 1.0 x^ 1 - 4.0 ) }

VAR
  i   : WORD;
  val : REAL;
  exp : WORD;
  s1  : STRING;
  Code: WORD;

BEGIN { CheckSyntax }
  Code := 0;
  i    := 1;
  REPEAT
    { s[]= (+3.5x^2-1.0x^1-4.0x^0)
      s[]=  123456789                  }
    IF Length (s) < 8 THEN BEGIN
      WriteLn ('ERROR: No hay suficientes argumentos');
      System.RunError( 213 { Collection index out of range } );
    END;

    { Incluye chequeo de signo }
    s1 := s[i] + s[i+1] + s[i+2] + s[i+3];
    System.Val (s1, val, Code);

    { Chequear primer coeficiente, con todo y su signo }
    IF (Code <> 0)     THEN BEGIN
      CheckSyntax := i+Code;
      EXIT;
    END;

    { Chequear coeficiente literal }
    IF (s[i+4] <> 'x') THEN BEGIN
      CheckSyntax := i+5;
      EXIT;
    END;

    { Chequear smbolo de exponenciacin }
    IF (s[i+5] <> '^') THEN BEGIN
      CheckSyntax := i+6;
      EXIT;
    END;

    { Chequear potencia }
    System.Val (s[i+6], exp, Code);
    IF (Code <> 0) THEN BEGIN
      CheckSyntax := i+Code+5;
      EXIT;
    END;
    i := i + 7;
  UNTIL s[i] = ')';

  CheckSyntax := Code;
END; { CheckSyntax }


PROCEDURE TPoly.Read(        { EXPORT }         { ADH }
  {?} VAR F: TEXT            { de donde leer }
);
{ RESULTADO
  Lee del archivo de texto F el valor de "SELF". }
{ REQUIERE
  - En general, SELF debe haber sido escrito en F
    usando Print(F). }
VAR
  s    : STRING;      { hilera de trabajo }
  sVal : STRING[19];  { valor numrico actual }
  sCoef: REAL;        { coeficiente }
  sExp : WORD;        { exponente }

  i  : WORD;
  ch : CHAR;
  errPos: INTEGER;    { usado por System.Val() }
BEGIN { TPoly.Read }
  Check_Ok(SELF, 'Read');

{ NOTA DE IMPLEMENTACION
  Para esta implementacin TPoly.Read() necesita que la
  entrada est en un formato que incluye el signo de cada
  coeficiente, y el grado de cada trmino, como se ve en
  este ejemplo:   ( + 3.5 x^ 2 - 1.0 x^ 1 - 4.0 )
  - Esto quiere decir que TPoly.Read() no puede leer una
    hilera que haya sido obtenida por medio de
    TPoly.To_STRING().
  - O sea, que esta implementacin NO CUMPLE con la
    especificacin, y por lo tanto es INCORRECTA. }

  { lee la entrada completa y chequea su sintaxis }

  s := '';
  REPEAT
    System.Read(F, ch);
    IF ch <> ' ' THEN BEGIN { se brinca los blancos }
      s := s+ch;
    END;
  UNTIL ch = ')';

{
  WriteLn ('Tira sin espacios:',  s);
  WriteLn ('Posicin         :', '1234567...');
}

  Zero;    { inicializa al polinomio }

  { traga hasta encontrar parntesis de cierre }
  i := 2;  { se brinca primer parntesis }
  WHILE s[i] <> ')' DO BEGIN
   { s[]= "(+3.5x^2-1.0x^1-4.0x^0)"
     s[]=  123456789  <= Pos()        }

    { saca al coeficiente }
    sVal := '';
    WHILE (s[i]<>'x') AND (s[i]<>')') DO BEGIN
      sVal := sVal+s[i];
      INC(i);
    END;
    System.Val(sVal, sCoef, errPos);

    { saca al exponente }
    sExp := 0;
    IF s[i]<>')' THEN BEGIN
      i := i+2;             { se brinca x^ }

      sVal := '';
      WHILE ('0' <= s[i]) AND (s[i] <= '9') DO BEGIN
        sVal := sVal+s[i];
        INC(i);
      END;

      System.Val(sVal, sExp, errPos);
    END;

    IF Coef(sExp) = 0.0 THEN BEGIN
      SetTerm(sCoef, sExp);
    END
    ELSE BEGIN
      { ERROR: el mismo trmino aparece dos veces }
      System.RunError( 106 { Invalid numeric format } );
    END;
  END;

  { esto sobra }
  Adjust_Degree;
END;  { TPoly.Read }


PROCEDURE TPoly_Read(        { EXPORT }         { ADH }
  {?} VAR SLF: TPoly;        { SELF }
  {?} VAR F  : TEXT          { de donde leer }
);
{ RESULTADO
  Lee del archivo de texto F el valor de "SELF". }
{ REQUIERE
  En general, SELF debe haber sido escrito en F
  usando Print(F). }
VAR
  ch : CHAR;
  val: REAL;
  exp: WORD;
BEGIN { TPoly.Read }
  Check_Ok(SLF, 'Read');

{ NOTA DE IMPLEMENTACION
  Para esta implementacin TPoly.Read() necesita que la
  entrada est en un formato que incluye el signo de cada
  coeficiente, y el grado de cada trmino, como se ve en
  este ejemplo:   ( + 3.5 x^ 2 - 1.0 x^ 1 - 4.0 )
  - Esto quiere decir que TPoly.Read() no puede leer una
    hilera que haya sido obtenida por medio de
    TPoly.To_STRING().
  - O sea, que esta implementacin NO CUMPLE con la
    especificacin, y por lo tanto es INCORRECTA. }

  {$I+} { explota con errores de entrada }

  { se brinca el primer parntesis }
  REPEAT
    System.Read(F, ch);
  UNTIL ch = '(';

  { traga hasta encontrar parntesis de cierre }
  WHILE ch <> ')' DO BEGIN
    { +  ( + 3.5 x^ 2 - 1.0 x^ 1 - 4.0 ) }

    System.Read(F, ch);        { ' ' }  { ' ' } { ' ' }
    System.Read(F, ch);        { '+' }  { '-' } { '-' }
    System.Read(F, val);       { 3.5 }  { 1.0 } { 4.0 }
    IF ch='-' THEN BEGIN
      val := -val;
    END;
    System.Read(F, ch);        { ' ' }  { ' ' }  { ' ' }
    System.Read(F, ch);        { 'x' }  { 'x' }  { ')' }

    exp := 0;
    IF ch<>')' THEN BEGIN
      System.Read(F, ch);      { '^' }  { '^' }
      System.Read(F, exp);     {  2  }  { '1' }
    END;

    SLF.SetTerm(val, exp);
  END;

  SLF.Adjust_Degree;
END;  { TPoly.Read }


PROCEDURE TPoly.Print(       { EXPORT }         { ADH }
  {?} VAR F: TEXT            { adonde escribir }
);
{ RESULTADO
  Imprime a "SELF" en un archivo de texto, de forma
  que sea legible usando un editor de texto.
  - El formato de escritura es (2/3). }
BEGIN { TPoly.Print }
  Check_Ok(SELF, 'Print');

  System.Write(F, '(', To_STRING,')');
END;  { TPoly.Print }

PROCEDURE TPoly.Zero;        { EXPORT }         { ADH }
{ RESULTADO
  Hace que el valor del polinomio sea cero. }
BEGIN { Zero }
  Check_Ok(SELF, 'Zero');

  _coef[0] := 0;
  _grad   :=  0;
END;  { Zero }

PROCEDURE TPoly.SetTerm(     { EXPORT }         { ADH }
  {+}     c : REAL;          { Coeficiente para p }
  {+}     d : WORD           { Grado }
);
{ RESULTADO
  Fuerza a que el trmino "grado" de SELF tenga
  coeficiente "coef".
  - Si coef=0.0 y SELF.Degree=grado, entonces el
    grado del SELF queda reducido, y puede
    eventualmente hacerse cero. }
VAR
  i : WORD;
BEGIN { SetTerm }
  Check_Ok(SELF, 'SetTerm');

  _coef[d] := c;

  IF d > _grad THEN BEGIN
    FOR i := d-1 DOWNTO _grad+1 DO BEGIN
      _coef[i] := 0.0;
    END;
    _grad := d;
  END;

  Adjust_Degree;
END;  { SetTerm }

FUNCTION TPoly.Degree        { EXPORT }         { ADH }
  {>>>>>>>>} : WORD;
{ RESULTADO
  Retorna el grado del polinomio SELF, o sea que
  retorna el mayor exponente del polinomio que
  tiene un coeficiente diferente de cero.
  - Si el polinomio es cero, retorna cero. }

{ EJEMPLO
  polinomio               grado
  x^3 + 2x^2                3   }
BEGIN { Degree }
  Check_Ok(SELF, 'Degree');

  Degree := _grad;
END;  { Degree }

FUNCTION TPoly.Coef(         { EXPORT }         { ADH }
  {+}     exp: WORD          { Exponente del coeficiente }
) {>>>>>>>>>}: REAL;
{ RESULTADO
  Devuelve el coeficiente de grado "exp" del polinomio SELF.
  - Si "exp" es ms grande que el grado del polinomio entonces
    retorna cero. }
{ EJEMPLO
      p(x)        exp       p.Coef(exp)
  x^3 + 2x + 1     0              1
  x^3 + 2x + 1     1              2
  x^3 + 2x + 1     2              0
  x^3 + 2x + 1     3              1
  x^3 + 2x + 1     4              0      }

BEGIN { TPoly.Coef }
  Check_Ok(SELF, 'Coef');

  IF exp <= _grad THEN BEGIN
    Coef := _coef[exp];
  END
  ELSE BEGIN
    Coef := 0.0;
  END;
END;  { TPoly.Coef }

PROCEDURE TPoly.Add(         { EXPORT }         { ADH }
  {+} VAR o : TPoly          { sumando }
);
{ RESULTADO
  Calcula SELF := SELF + o. }
VAR
  i, min : WORD;
BEGIN { TPoly.Add }
  Check_Ok(SELF, 'Add');
  Check_Ok(o,    'Add');

  IF _grad < o._grad THEN BEGIN
    min := _grad;
    _grad := o._grad;

    { suma los primeros trminos, que estn en los dos }
    FOR i := 0 TO min DO BEGIN
      _coef[i] := _coef[i]+o._coef[i];
    END;

    { agrega los trminos que slo aparecen en "o" }
    FOR i := min+1 TO o._grad DO BEGIN
      _coef[i] := o._coef[i];
    END;
  END
  ELSE BEGIN
    min := o._grad;

    FOR i := 0 TO min DO BEGIN
      _coef[i] := _coef[i]+o._coef[i];
    END;
  END;

  { Averigue si la operacin anul al polinomio }
  Adjust_Degree;
END;  { TPoly.Add }

PROCEDURE TPoly.Substract(   { EXPORT }         { ADH }
  {+} VAR o : TPoly          { sustraendo }
);
{ RESULTADO
  Calcula SELF := SELF - o. }
VAR
  i,j : WORD;
  tmp : TPoly; { temporal para hacer la multiplicacin }
BEGIN { TPoly.Substract }
  Check_Ok(SELF, 'Substract');
  Check_Ok(o,    'Substract');

  tmp.Init;
  tmp.Copy(o);
  tmp.Minus;
  Add(tmp);
  tmp.Done;

  {$IFDEF Never_RUN} {$IFNDEF Never_RUN}

    o.Minus;  { le vuelve el signo al otro }
    Add(o);
    o.Minus;  { lo restaura }

  { Esta otra implementacin tambin funciona, pero 
    tiene el problema de que cambia por un momento
    el valor de la variable "o", en contra de lo que
    dice la especificacin.
    
    En un programa secuencial cualquiera esto no es
    problema, pero si Poly.pas se usa en un programa
    concurrente, entonces el cambio de valor que por
    un momento "o" sufre puede hacer que el programa
    sea incorrecto. Por eso es mejor jugar a lo seguro
    y NO cambiar el valor de una variable si uno as
    lo promete en la especificacin. }
  {$ENDIF}{$ENDIF}
   
END;  { TPoly.Substract }

PROCEDURE TPoly.Multiply(    { EXPORT }         { ADH }
  {+} VAR o : TPoly          { multiplicando }
);
{ RESULTADO
  Calcula SELF := SELF * o. }
{ REQUIERE
  Degree()+o.Degree() <= Poly.Max_Grado. }
VAR
  i,j : WORD;
  tmp : TPoly; { temporal para hacer la multiplicacin }
BEGIN { TPoly.Multiply }
  Check_Ok(SELF, 'Multiply');
  Check_Ok(o,    'Multiply');

  tmp.Init;

  tmp._grad := _grad + o._grad;
  FOR i := 0 TO tmp._grad DO BEGIN
    tmp._coef[i] := 0;
  END;

  FOR i := 0 TO _grad DO BEGIN
    FOR j := 0 TO o._grad DO BEGIN
      tmp._coef[i+j] :=
        tmp._coef[i+j] + Coef(i) * o.Coef(j);
    END;
  END;

  { Averigue si la operacin anul al polinomio }
  tmp.Adjust_Degree;

  { copia el resultado }
  Copy(tmp);
  tmp.Done;
END;  { TPoly.Multiply }

PROCEDURE Divide_JBV(        { LOCAL }          { JBV }
  {+} VAR p,                 { SELF }
  {+}     q : TPoly;         { divisor }
  {?} VAR d : TPoly;
  {?}     r : TPoly          { p(x) = q(x) * d(x) + r(x) }
);
{ RESULTADO
  Calcula la divisin del polinomio p(x) sobre q(x), y
  retorna en d(x) el resultado de la divisin, y en
  r(x) el resto, de forma que se cumpla que
               p(x) = q(x) * d(x) + r(x).
  - Los valores anteriores de "d" y "r" se pierden.
  - El resultado calculado es correcto an si "r" es
    SELF o "q". }
{ REQUIERE
  - q <> 0.   }

{ PROGRAMADOR  JBV ==> Jorge Alonso Betancourth Vega }

VAR
  coefp  : REAL;
  coefq  : REAL;
  valor  : REAL;  { parte entera de la divisin del }
                  { coeficiente del primer trmino de p y q }
  comp   : WORD;
  gradoq : WORD;
  grado  : WORD;

  temp   : Poly.TPoly;
  temp2  : Poly.TPoly;
BEGIN { Divide_JBV }
  { Parece que esta implementacin es incorrecta }
  System.RunError( 200 { Division by zero } );

  d.Clear;
  r.Clear;

  IF p.Degree < q.Degree THEN BEGIN
    r.Copy(p);
    EXIT;
  END;

  temp.Init;
  temp2.Init;

  gradoq := q.Degree;
  coefq  := q.Coef(gradoq);
  comp   := p.Degree;
  temp2.Copy(p);
  WHILE comp >= q.Degree DO BEGIN
    coefp := temp2.Coef(comp);
    valor := coefp/coefq;
    grado := comp - q.Degree;
    d.SetTerm(valor,grado);

    temp.Copy(q);
    temp.Multiply(d);

    r.Copy(temp2);
    r.Substract(temp);
    comp := r.Degree;
    temp2.Copy(r);
  END;

  temp.Done;
  temp2.Done;
END;  { Divide_JBV }

PROCEDURE Divide_RAM(        { LOCAL }          { RAM }
  {+} VAR p,                 { SELF }
  {+}     q : TPoly;         { divisor }
  {?} VAR d : TPoly;
  {?}     r : TPoly          { p(x) = q(x) * d(x) + r(x) }
);
{ RESULTADO
  Calcula la divisin del polinomio p(x) sobre q(x), y
  retorna en d(x) el resultado de la divisin, y en
  r(x) el resto, de forma que se cumpla que
               p(x) = q(x) * d(x) + r(x).
  - Los valores anteriores de "d" y "r" se pierden.
  - El resultado calculado es correcto an si "r" es
    SELF o "q". }
{ REQUIERE
  - q <> 0.   }

{ NOTA
  - Este algoritmo se hizo con base al trabajo de:
  PROGRAMADOR  JBV ==> Jorge Alonso Betancourth Vega  }
{ PROGRAMADOR  RAM ==> [950127] Ricardo Alvarado Mena }

VAR
  coefp  : REAL;
  coefq  : REAL;
  valor  : REAL;  { parte entera de la divisin del }
                  { coeficiente del primer trmino de p y q }
  comp   : WORD;
  gradoq : WORD;
  grado  : WORD;

  temp   : Poly.TPoly;
  temp2  : Poly.TPoly;
BEGIN { Divide_RAM }
  { Parece que esta implementacin slo }
  { funciona si el grado de "q" es 1.   }
  d.Clear;
  r.Clear;

  IF p.Degree < q.Degree THEN BEGIN
    r.Copy(p);
    EXIT;
  END;

  temp.Init;
  temp2.Init;

  gradoq := q.Degree;
  coefq  := q.Coef(gradoq);
  comp   := p.Degree;
  temp2.Copy(p);
  WHILE comp >= gradoq DO BEGIN
    coefp := temp2.Coef(comp);
    valor := coefp/coefq;
    grado := comp - gradoq;
    d.SetTerm(valor,grado);

    temp.Clear;
    temp.SetTerm(valor,grado);
    temp.Multiply(q);

    r.Copy(temp2);
    r.Substract(temp);
    comp := r.Degree;
    temp2.Copy(r);
  END;

  temp.Done;
  temp2.Done;
END;  { Divide_RAM }

PROCEDURE Divide_ACS(        { EXPORT }            { ACS }
  {+} VAR p,                 { SELF }
  {+}     q : TPoly;         { divisor }
  {?} VAR d,                 { resultado }
  {?}     r : TPoly          { p(x) = q(x) * d(x) + r(x) }
);
{ RESULTADO
  Calcula la divisin del polinomio p(x) sobre q(x), y
  retorna en d(x) el resultado de la divisin, y en
  r(x) el resto, de forma que se cumpla que
               p(x) = q(x) * d(x) + r(x).
  - Los valores anteriores de "d" y "r" se pierden.
  - El resultado calculado es correcto an si "r" es
    "p" o "q". }
{ REQUIERE
  - q <> 0.   }

{ NOTA
  - La operacin se realiza utilizando el algoritmo de
    Divisin Sintctica Generalizada descrito en:

    Ral Alvarado; " Una generalizacin de la Divisin
    Sinttica", Revista Ingeniera, Vol 3 #1, pg 21-27;
    Universidad de Costa Rica; 1992.
  - Este algoritmo se hizo con base al trabajo de:
  PROGRAMADOR  VGM ==> [931713] Vctor Guevara Mass   }
{ PROGRAMADOR  ACS ==> [931192] Alessandro Cordero Salazar }

CONST
  M_G = Poly.Max_Grado;
TYPE
  TMatriz   = ARRAY[1..M_G, 1..M_G] OF REAL;
  TDivisor  = ARRAY[1..M_G]         OF REAL;

VAR
  r1       : REAL;
  Matriz   : TMatriz;   { El maximo dividendo debe tener grado 24 }
  divisor  : TDivisor;
  i,j,k,
  pDeg,qDeg,
  aux,
  col,cont : WORD ;
BEGIN { Divide_ACS }
  IF q.Is_Zero THEN BEGIN
    System.RunError( 200 { Division by zero } );
    EXIT;
  END

  ELSE IF p.Degree < q.Degree THEN BEGIN
    { El grado del divisor es mayor que el grado del dividendo }
    r.Copy(p);
    d.Zero;            { p(x) = q(x) * d(x) + r(x) }
    EXIT;
  END;

  { Se pone la matriz y el vector en 0 }
  FOR i := 1 TO M_G DO BEGIN
    FOR k := 1 TO M_G DO BEGIN
      Matriz[i,k] := 0.0;
    END;
    divisor[i] := 0.0;
  END;
  pDeg := p.Degree;
  qDeg := q.Degree;

  { Se llena la primera fila de la matriz }
  k := 1;
  FOR i := pDeg DOWNTO 0 DO BEGIN
    Matriz[1,k] := p.Coef(i);
    INC(k);
  END;

  { Se llena el vector divisor con q }
  k := 1;
  FOR i := (qDeg - 1) DOWNTO 0 DO BEGIN
    { Aunque no sean mnicos }
    divisor[k] := ((-q.Coef(i))/q.Coef(qDeg));
    INC(k);
  END;

  col  := 2;
  j    := 1;
  cont := qDeg;

 { se llena el primer resultados }
  Matriz[qDeg+2,1] := Matriz[1,1];

  REPEAT
    aux := 2;
    k   := col;

   { Utiliza a la matriz de resultados }
    FOR i := 1 TO qDeg DO BEGIN
      Matriz[aux,k] := Matriz[qDeg+2,j] * divisor[i];
      INC(aux);
      INC(k);
    END;

    { Suma las columnas de la matriz }
    FOR i := 1 TO (qDeg + 1 ) DO BEGIN
      r1 := Matriz[i,col];
      Matriz[qDeg+2,col] := (r1 + Matriz[qDeg+2,col] );
    END;
    INC(col);
    INC(cont);
    INC(j);
  UNTIL (cont = pDeg + 1) ;

  { suma las columnas que no fueron sumadas }
  WHILE col <= (pDeg + 1) DO BEGIN
    FOR i := 1 TO (qDeg + 1 ) DO BEGIN
      r1 := Matriz[i,col];
      Matriz[qDeg+2,col] := (r1 + Matriz[qDeg+2,col] );
    END;
    INC(col);
  END;

  { Prepara los polinomios temporales }
  r.Clear;
  d.Clear;

  j := 1;
  k := (pDeg - qDeg);

  { Coloca en TmpD el polinomio resultante }
  FOR i := ((pDeg - qDeg)  + 1 ) DOWNTO 1  DO BEGIN
    r1 :=  (Matriz[qDeg+2,j]/q.Coef(qDeg));
    d.SetTerm(r1,k);
    INC(j);
    DEC(k) ;
  END;
  j := 0;

  { Almacena en tmpR el residuo de la divisin.}
  FOR i := (pDeg + 1) DOWNTO ((pDeg - qDeg) + 2) DO BEGIN
    r1 :=  (Matriz[qDeg+2,i]/q.Coef(qDeg));
    r.SetTerm(r1,j);
    INC(j);
  END;

END;  { Divide_ACS }

PROCEDURE Divide_KMF(        { EXPORT }            { KMF }
  {+} VAR p,                 { SELF }
  {+}     q : TPoly;         { divisor }
  {?} VAR d,                 { resultado }
  {?}     r : TPoly          { p(x) = q(x) * d(x) + r(x) }
);
{ RESULTADO
  Calcula la divisin del polinomio p(x) sobre q(x), y
  retorna en d(x) el resultado de la divisin, y en
  r(x) el resto, de forma que se cumpla que
               p(x) = q(x) * d(x) + r(x).
  - Los valores anteriores de "d" y "r" se pierden.
  - El resultado calculado es correcto an si "r" es
    "p" o "q". }
{ REQUIERE
  - q <> 0.   }

{ NOTA
  - La operacin se realiza utilizando el algoritmo de
    Divisin Sintctica Generalizada descrito en:

    Ral Alvarado; " Una generalizacin de la Divisin
    Sinttica", Revista Ingeniera, Vol 3 #1, pg 21-27;
    Universidad de Costa Rica; 1992.
  - Este algoritmo se hizo con base al trabajo de:
  PROGRAMADOR  VGM ==> [931713] Vctor Guevara Mass   }
{ PROGRAMADORA KMF ==> [942010] Kattia Madrigal Fallas }

CONST
  M_G = Poly.Max_Grado;
TYPE
  TMatriz   = ARRAY[1..M_G, 1..M_G] OF REAL;
  TDivisor  = ARRAY[1..M_G]         OF REAL;

VAR
  r1, r2   : REAL;
  Matriz   : TMatriz;   { El maximo dividendo debe tener grado 50 }
  divisor  : TDivisor;
  tmpR,
  tmpD     : TPoly;
  cons,
  i,j,
  x,y,z,
  aux1,
  aux2,
  aux3,
  columna,
  contador,
  exp       : WORD ;
BEGIN { Divide_KMF }
  IF q.Is_Zero THEN BEGIN
    System.RunError( 200 { Division by zero } );
    EXIT;
  END

  ELSE IF p.Degree < q.Degree THEN BEGIN
    { El grado del divisor es mayor que el grado del dividendo }
    r.Copy(p);
    d.Zero;            { p(x) = q(x) * d(x) + r(x) }
    EXIT;
  END;

  { Se pone la matriz y el vector en 0 }
  FOR i := 1 TO M_G DO BEGIN
    FOR z := 1 TO M_G DO BEGIN
      Matriz[i,z] := 0.0;
    END;
    divisor[i] := 0.0;
  END;

  { Se llena la primera fila de la matriz }
  cons := 1;
  FOR i := p.Degree DOWNTO 0 DO BEGIN
    Matriz[1,cons] := p.Coef(i);
    INC(cons);
  END;

  { Se llena el vector divisor con q }
  cons := 1;
  x := q.Degree;
  FOR i := (x - 1) DOWNTO 0 DO BEGIN
    { Aunque no sean mnicos }
    divisor[cons] := ((-q.Coef(i))/q.Coef(x));
    INC(cons);
  END;

  IF q.Degree <= p.Degree THEN BEGIN
     columna  := 2;
     aux2     := 2;
     aux3     := 2;
     y        := 1;
     j        := (q.Degree + 2);
     contador := q.Degree;

    { se llena el primer resultados }
     Matriz[j,1] := Matriz[1,1];

     REPEAT
       aux1 := aux2;
       z    := aux3;

      { Utiliza a la matriz de resultados }
       FOR i := 1 TO q.Degree DO BEGIN
         Matriz[aux1,z] := Matriz[j,y] * divisor[i];
         INC(aux1);
         INC(z);
       END;

       { Suma las columnas de la matriz }
       FOR i := 1 TO (q.Degree + 1 ) DO BEGIN
         r1 := Matriz[i,columna];
         Matriz[j,columna] := (r1 + Matriz[j,columna] );
       END;
       INC(columna);
       INC(aux3);
       INC(contador);
       INC(y);
     UNTIL (contador = p.Degree + 1) ;

     { suma las columnas que no fueron sumadas }
     WHILE columna <= (p.Degree + 1) DO BEGIN
       FOR i := 1 TO (q.Degree + 1) DO BEGIN
         r1 := Matriz[i,columna];
         Matriz[j,columna] := (r1 + Matriz[j,columna] );
       END;
       INC(columna);
     END;

     { Prepara los polinomios temporales }
     tmpR.Init;
     tmpD.Init;
     tmpR.Clear;
     tmpD.Clear;

     y := 1;
     exp := (p.Degree - q.Degree);

     { Coloca en TmpD el polinomio resultante }
     FOR i := ((p.Degree - q.Degree)  + 1 ) DOWNTO 1  DO BEGIN

       r2 :=  (Matriz[j,y]/q.Coef(x));
       tmpD.SetTerm(r2,exp);
       INC(y);
       DEC(exp) ;
     END;
     y := 0;

     { Almacena en tmpR el residuo de la divisin.}
     FOR i := (p.Degree + 1) DOWNTO ((p.Degree - q.Degree) + 2) DO BEGIN
       exp := y;
       r2 :=  (Matriz[j,i]); {/q.Coef(x));}
       tmpR.SetTerm(r2,exp);
       INC(y);
     END;

     { Pone los resultados, despues de borrar }
     d.Move(tmpD);
     r.Move(tmpR);

     { Destruye los polinomios temporales }
     tmpR.Done;
     tmpD.Done;
  END;
END;  { Divide_KMF }

PROCEDURE TPoly.Divide(      { EXPORT }         { ADH }
  {+} VAR o : TPoly          { dividendo }
);
{ RESULTADO
  Calcula SELF := SELF DIV o. }
{ REQUIERE
  o <> 0. }
VAR
  d,r : TPoly;
BEGIN { TPoly.Divide }
  Check_Ok(SELF, 'Divide');
  Check_Ok(o,    'Divide');

  d.Init;
  r.Init;

  Poly.Divide_ACS(SELF, o, d, r);
  Copy(d);

  r.Done;
  d.Done;
END;  { TPoly.Divide }

PROCEDURE TPoly.Modulo(      { EXPORT }         { ADH }
  {+} VAR o : TPoly          { dividendo }
);
{ RESULTADO
  Calcula SELF := SELF MOD o. }
{ REQUIERE
  o <> 0. }
VAR
  d,r : TPoly;
BEGIN { TPoly.Modulo }
  Check_Ok(SELF, 'Modulo');
  Check_Ok(o,    'Modulo');

  d.Init;
  r.Init;

  Poly.Divide_ACS(SELF, o, d, r);
  Copy(r);

  r.Done;
  d.Done;
END;  { TPoly.Modulo }

PROCEDURE TPoly.Minus;       { EXPORT }         { ADH }
{ RESULTADO
  Le cambia el signo a SELF.
  - Para esto, el cambia el signo a cada uno de sus
    coeficientes. }
VAR
  i : WORD;
BEGIN { TPoly.Minus }
  Check_Ok(SELF, 'Minus');

  { slo toca a los trminos que son importantes }
  FOR i := 0 TO _grad DO BEGIN
    _coef[i] := -_coef[i];
  END;
END;  { TPoly.Minus }

FUNCTION  TPoly.Is_ZERO      { EXPORT }         { ADH }
  {>>>>>>>>}: BOOLEAN;       { p=0 ? }
{ RESULTADO
  Retorna TRUE si SELF es el polinomio cero. }
BEGIN { TPoly.Is_ZERO }
  Check_Ok(SELF, 'Is_ZERO');

  Is_ZERO :=
    (_grad = 0)
      AND
    (_coef[0] = 0.0);
END;  { TPoly.Is_ZERO }

FUNCTION  TPoly.Eval(        { EXPORT }         { ADH }
  {+}     x : REAL
) {>>>>>>>} : REAL;          { p(x) }
{ RESULTADO
  Evala al polinomio SELF en la variable "x".
  - Para calcular p(x) se usa la Regla de Horner. }
VAR
  i   : WORD;
  sum : REAL;
BEGIN { TPoly.Eval }
  Check_Ok(SELF, 'Eval');

  sum := 0;
  FOR i := _grad DOWNTO 0 DO BEGIN
    sum := x * sum + _coef[i];
  END;

  Eval := sum;
END;  { TPoly.Eval }


FUNCTION  Coef_STRING(       { LOCAL }          { ADH }
  {+} r : REAL
) {>>>} : STRING;
{ RESULTADO
  Convierte el nmero "r" a su representacin
  de hilera, pero cuidando de eliminar
  la parte decimal si sta es cero.
  - Coef_STRING(1.0) = "1"
  - Coef_STRING(1.5) = "1.5"  }
VAR
  s : STRING[30];
BEGIN { Coef_STRING }
  IF Frac(r) = 0.0 THEN BEGIN
    System.Str(r:0:0, s);
  END
  ELSE BEGIN
    System.Str(r:0:1, s);
  END;

  Coef_STRING := s;
END;  { Coef_STRING }

FUNCTION  TPoly.To_STRING    { EXPORT }         { ADH }
  {>>>>>>>>>>} : STRING;     { Hilera imagen de p }
{ RESULTADO
  Trasforma el polinomio SELF a una hilera, aunque
  no incluye todos los decimales.
  - No incluye los trminos de coeficiente cero.
  - Es muy til para desplegar a SELF en la pantalla.
  - La hilera retornada tiene una capacidad mxima de
    SizeOf(STRING)-1 caracteres. }
{ EJEMPLOS
  x^2+x
  x^3-2x^2+1.4
  11x^3-2       }
VAR
  i : WORD;
  h : STRING;
  s : STRING[30];
BEGIN { TPoly.To_STRING }
  Check_Ok(SELF, 'To_STRING');

  h := '';

  FOR i := _grad DOWNTO 0 DO BEGIN

    IF _coef[i] > 0.0 THEN BEGIN
      IF (_coef[i] = 1.0) AND (i<>0) THEN BEGIN
        s := '';
      END
      ELSE BEGIN
        s := Coef_STRING(_coef[i]);
      END;

      IF i <> _grad THEN BEGIN
        h := h + '+' + s;
      END
      ELSE BEGIN
        h := h + s;
      END;

      IF i>1 THEN BEGIN
        System.Str(i:0, s);
        h := h + 'x^' + s;
      END
      ELSE IF i = 1 THEN BEGIN
        h := h + 'x';
      END;
    END
    ELSE IF _coef[i] < 0.0 THEN BEGIN
      IF (_coef[i] = -1.0) AND (i<>0) THEN BEGIN
        s := '-';
      END
      ELSE BEGIN
        s := Coef_STRING(_coef[i]);
      END;

      h := h + s;

      IF i>1 THEN BEGIN
        System.Str(i:0, s);
        h := h + 'x^' + s;
      END
      ELSE IF i = 1 THEN BEGIN
        h := h + 'x';
      END;
    END;
  END;

  IF h = '' THEN BEGIN
    h := '0';
  END;

  To_STRING := h;
END;  { TPoly.To_STRING }

PROCEDURE TPoly.From_STRING( { EXPORT }         { ADH }
  {+} s : STRING             { de dnde dar valor }
);
{ RESULTADO
  Toma el contenido de la hilera "s" y lo usa para darle un
  nuevo valor al polinomio.
  - Presumiblemente el valor de "s" fue obtenido en una
    invocacin previa a TPoly.To_STRING().
  - "s" puede contener algunos blancos para separar los
     trminos. }
{ EJEMPLOS
  s = 'x^2+x'
  s = 'x^3-2x^2+1.4'
  s = '11x^3-2'       }
BEGIN { TPoly.From_STRING }
  Check_Ok(SELF, 'From_STRING');

  { NO IMPLEMENTADO }
  System.RunError( 211 { Call to abstract method } );
END;  { TPoly.From_STRING }



{ Comentario eliminado: }
{ ====================  }
  { No se inicializa a tmp porque TPoly.Init() lo }
  { nico que hace es poner en cero al polinomio, }
  { y no inicializa memoria dinmica. Adems,     }
  { Copy() reinicializa todos los campos del Rep. }

  { Tampoco hace falta destruir a "tmp" porque }
  { TPoly.Done() realmente no hace nada        }

  { OJO:                                             }
  { Si la implementacin de TPoly cambia, es posible }
  { que sea necesario entonces construir y destruir  }
  { adecuadamente a "tmp".                           }

BEGIN { Poly }
  System.Randomize;
  Initialized_TPoly := System.Random(MaxInt);
  Destroyed_TPoly   := NOT Initialized_TPoly;
END.  { Poly }

{ EOF: Poly.pas }
